@@ -53,7 +53,7 @@ sub create_druid_page {
 		Gtk2->main_quit;
 		return 1;
 	});
-	$this->druid_page->signal_connect("cancel", sub { exit });
+	$this->druid_page->signal_connect("cancel", sub { exit 1 });
 	$this->druid_page->show;
 	$this->druid->append_page($this->druid_page);
 	$this->druid->set_page($this->druid_page);
@@ -87,6 +87,7 @@ sub init {
 	$this->SUPER::init(@_);
 	$this->interactive(1);
 	$this->capb('backup');
+	$this->need_tty(0);
 	
 	$this->win(Gtk2::Window->new("toplevel"));
 	$this->win->set_position("center");
@@ -94,9 +95,20 @@ sub init {
 	my $hostname = `hostname`;
 	chomp $hostname;
 	$this->win->set_title(to_Unicode(sprintf(gettext("Debconf on %s"), $hostname)));
-	$this->win->signal_connect("delete_event", sub { exit });
+	$this->win->signal_connect("delete_event", sub { exit 1 });
 	
-	$this->logo(Gtk2::Gdk::Pixbuf->new_from_file("/usr/share/pixmaps/debian-logo.png"));
+	my $distribution='';
+	if (system('type lsb_release >/dev/null 2>&1') == 0) {
+		$distribution=lc(`lsb_release -is`);
+		chomp $distribution;
+	} elsif (-e '/etc/debian_version') {
+		$distribution='debian';
+	}
+
+	my $logo="/usr/share/pixmaps/$distribution-logo.png";
+	if (-e $logo) {
+		$this->logo(Gtk2::Gdk::Pixbuf->new_from_file($logo));
+	}
 	
 	$this->druid(Gnome2::Druid->new);
 	$this->druid->show;
@@ -149,6 +161,56 @@ sub go {
 	return 1;
 }
 
+sub progress_start {
+	my $this=shift;
+	$this->SUPER::progress_start(@_);
+
+	my $element=$this->progress_bar;
+	$this->druid_page->vbox->pack_start($element->hbox, $element->fill, $element->expand, 0);
+	$this->druid_page->set_title(to_Unicode($this->title));
+	# TODO: no backup support yet
+	$this->druid->set_buttons_sensitive(0, 0, 1, 1);
+	$this->win->show;
+
+	while (Gtk2->events_pending) {
+		Gtk2->main_iteration;
+	}
+}
+
+sub progress_set {
+	my $this=shift;
+
+	my $ret=$this->SUPER::progress_set(@_);
+
+	while (Gtk2->events_pending) {
+		Gtk2->main_iteration;
+	}
+
+	return $ret;
+}
+
+sub progress_info {
+	my $this=shift;
+	my $ret=$this->SUPER::progress_info(@_);
+
+	while (Gtk2->events_pending) {
+		Gtk2->main_iteration;
+	}
+
+	return $ret;
+}
+
+sub progress_stop {
+	my $this=shift;
+	$this->SUPER::progress_stop(@_);
+
+	while (Gtk2->events_pending) {
+		Gtk2->main_iteration;
+	}
+
+	$this->create_druid_page();
+}
+
 =back
 
 =head1 AUTHOR
