@@ -103,6 +103,7 @@ sub new {
 		$this = fields::new($this);
 	}
 	$this->{template}=$template;
+
 	# Create a question in the db to go with it, unless
 	# one with the same name already exists. If one with the same name
 	# exists, it may be a shared question so we add the current owner
@@ -119,6 +120,7 @@ sub new {
 	# This is what actually creates the template in the db.
 	return unless $Debconf::Db::templates->addowner($template, $template, $type);
 
+	$Debconf::Db::templates->setfield($template, 'type', $type);
 	return $template{$template}=$this;
 }
 
@@ -198,7 +200,7 @@ sub load {
 
 			if ($field ne '') {
 				if (exists $data{$field}) {
-					die sprintf(gettext("Template #%s in %s has a duplicate field \"%s\" with new value \"%s\". Probably two templates are not properly seperated by a lone newline.\n"), $., $file, $field, $value);
+					die sprintf(gettext("Template #%s in %s has a duplicate field \"%s\" with new value \"%s\". Probably two templates are not properly separated by a lone newline.\n"), $., $file, $field, $value);
 				}
 				$data{$field}=$value;
 				$data{"extended_$field"}=$extended
@@ -424,15 +426,16 @@ sub AUTOLOAD {
 	no strict 'refs';
 	*$AUTOLOAD = sub {
 		my $this=shift;
-			if (@_) {
+		if (@_) {
 			return $Debconf::Db::templates->setfield($this->{template}, $field, shift);
 		}
 		
 		my $ret;
+
 		# Check to see if i18n and/or charset encoding should
 		# be used.
 		if ($Debconf::Template::i18n && @langs) {
-			my @fields = grep /^\Q$field\E/, $Debconf::Db::templates->fields($this->{template});
+			my @fields = grep /^\Q$field\E(?:[_.@]|$)/, $Debconf::Db::templates->fields($this->{template});
 			foreach my $lang (@langs) {
 				# First check for a field that matches the
 				# language and the encoding. No charset
@@ -445,7 +448,7 @@ sub AUTOLOAD {
 				# Failing that, look for a field that matches
 				# the language, and do charset conversion.
 				if ($Debconf::Encoding::charmap) {
-					foreach my $f (@fields) {
+					foreach my $f ($Debconf::Db::templates->fields($this->{template})) {
 						if ($f =~ /^\Q$field-$lang\E\.(.+)/) {
 							my $encoding = $1;
 							$ret = Debconf::Encoding::convert($encoding, $Debconf::Db::templates->getfield($this->{template}, lc($f)));
@@ -454,9 +457,25 @@ sub AUTOLOAD {
 					}
 				}
 			}
+		} elsif (not $Debconf::Template::i18n && $field !~ /-c$/i) {
+			# If i18n is turned off, try *-C first.
+			$ret=$Debconf::Db::templates->getfield($this->{template}, $field.'-c');
+			return $ret if defined $ret;
 		}
+
 		$ret=$Debconf::Db::templates->getfield($this->{template}, $field);
 		return $ret if defined $ret;
+
+		# If the user asked for *-C, fall back to the unadorned
+		# field. This allows *-C to be used to get untranslated
+		# data.
+		if ($field =~ /-c$/i) {
+			(my $plainfield = $field) =~ s/-c$//i;
+			$ret=$Debconf::Db::templates->getfield($this->{template}, $plainfield);
+			return $ret if defined $ret;
+			return '';
+		}
+
 		return '';
 	};
 	goto &$AUTOLOAD;
